#pragma once

#include <string>
#include <unordered_map>
#include <optional>

struct LatexMathSymbol
{
	enum class Type
	{
		RAW,
		TAG,
		RELATIONAL,
		RELATIONAL_TAG,
		ENCLOSING,
		ENCLOSING_TAG
	};

	enum class Tag
	{
		NONE,
		FRAC
	};

	LatexMathSymbol(Type type, const std::string& name, const std::string& unicode)
		:	mType(type),
			mName(name),
			mUnicode(unicode)
	{

	}

	Type mType{ Type::RAW };
	std::string mName;
	std::string mUnicode;
};

class LatexSymbolLookup
{
public:
	static std::optional<std::string> getSymbolUtf8(const std::string& tag);

	static std::optional<std::wstring> getSymbolUtf16(const std::string& tag);

	static std::optional<LatexMathSymbol::Tag> getKnownTag(const std::string& tag);

	static std::size_t getNumTagBodies(LatexMathSymbol::Tag tag)
	{
		switch (tag)
		{
		case LatexMathSymbol::Tag::FRAC:
			return 2;
		default:
			return 1;
		}
	}

private:
	static std::unordered_map<std::string, std::wstring> mSymbols;

	static std::unordered_map<std::string, LatexMathSymbol::Tag> mTags;
};